<?php
/**
 * EverestForms Salesforce Api Class.
 *
 * @package EverestForms\Salesforce\Api
 * @since  1.0.0
 */

namespace EverestForms\Salesforce\Api;

use WP_Error;

defined( 'ABSPATH' ) || exit;

/**
 * Class API.
 */
class Api {

	/**
	 * Api Version.
	 *
	 * @since 1.0.0
	 *
	 * @var String
	 */
	protected $version = '48.0';


	/**
	 * Connection.
	 *
	 * Contains All the information of a connection.
	 *
	 * @var array
	 */
	protected $connection = array();

	/**
	 * Create a new API instance
	 *
	 * @param string $connection Connection Array.
	 */
	public function __construct( $connection ) {

		// Set Connection Args.
		$this->connection = $connection;

		$this->client = ( new Auth( $connection ) )->get_client();
	}

	/**
	 * Send Request to Salesforce Api.
	 *
	 * @since 1.0.0
	 *
	 * @param string $resource Resource Path.
	 * @param array  $body Data.
	 * @param string $method Method.
	 *
	 * @return mixed
	 * @throws Exception Emits exception on requests failure.
	 */
	protected function make_request( $path, $data = null, $method ) {
		$path        = rtrim( $path, '/' );
		$request_url = "{$this->connection['instance_url']}/services/data/v{$this->version}/{$path}";

		if ( is_array( $data ) ) {
			$data = (object) $data;
		}

		if ( null !== $data ) {
			$data = wp_json_encode( $data );
		}

		$options = array(
			'headers' => array(
				'Content-Type' => 'application/json',
			),
			'body'    => $data,
		);

		$request = $this->client->getAuthenticatedRequest(
			$method,
			esc_url( $request_url ),
			$this->connection['access_token'],
			$options
		);

		$response = $this->client->getParsedResponse( $request );

		if ( ! is_array( $response ) ) {
			throw new Exception(
				'Invalid response received from Authorization Server. Expected JSON.'
			);
		}

		return $response;
	}

	/**
	 * Get All Fields.
	 *
	 * @since 1.0.0
	 *
	 * @param string $module_name Module Name.
	 */
	public function get_fields( $module_name ) {
		try {
			$response = $this->make_request( "sobjects/{$module_name}/describe", array(), 'GET' );
		} catch ( Exception $e ) {
			$result = $e->getMessage();
		}

		if ( ! is_array( $response ) || empty( $response['fields'] ) ) {
			return null;
		}

		$fields = $response['fields'];
		$result = [
			'required' => [],
			'optional' => [],
		];

		$types     = array_column( $fields, 'type', 'name' );
		$labels    = array_column( $fields, 'label', 'name' );
		$compounds = [];

		foreach ( $fields as $field ) {

			// Skip fields with those parameters (they are not available for filling).
			if ( ! $field['createable'] || $field['deprecatedAndHidden'] ) {
				continue;
			}

			$key = $this->is_required_field( $field ) ? 'required' : 'optional';

			if ( empty( $field['compoundFieldName'] ) ) {
				$result[ $key ][ $field['name'] ] = $field;
				continue;
			}

			// Compound - it's group of fields for Salesforce's Address, Name, Location.
			$compound_name  = $field['compoundFieldName'];
			$compound_group = [
				'name'            => $compound_name,
				'label'           => $labels[ $compound_name ],
				'type'            => $types[ $compound_name ],
				'compound_fields' => [],
			];

			// Mark all fields from one compound and with different types (required|optional) like required fields.
			if (
				! empty( $compounds[ $compound_name ] ) &&
				$key !== $compounds[ $compound_name ]
			) {
				if ( 'required' !== $key ) {
					$key = 'required';
				} else {
					$compound_group              = $result[ $compounds[ $compound_name ] ][ $compound_name ];
					$compounds[ $compound_name ] = $key;
				}
			}

			if ( empty( $result[ $key ][ $compound_name ] ) ) {
				$result[ $key ][ $compound_name ] = $compound_group;
				$compounds[ $compound_name ]      = $key;
			}

			$result[ $key ][ $compound_name ]['compound_fields'][] = $field;
		}
		return $result;
	}


	/**
	 * Send a request for creating a new object.
	 *
	 * @since 1.0.0
	 *
	 * @param string $name Object name.
	 * @param array  $args Request arguments.
	 *
	 * @return mixed
	 */
	public function create_object( $name, $args ) {

		try {
			$response = $this->make_request( "sobjects/{$name}", $args, 'POST' );
		} catch ( Exception $e ) {
			$response = $e->getMessage();
		}
		return $response;
	}

	/**
	 * Determine if it's a required field.
	 *
	 * @since 1.0.0
	 *
	 * @param array $field Salesforce field field/properties.
	 *
	 * @return bool
	 */
	public function is_required_field( $field ) {

		if ( empty( $field ) || ! is_array( $field ) ) {
			return false;
		}

		if ( ! isset( $field['defaultedOnCreate'], $field['nillable'] ) ) {
			return false;
		}

		return ! $field['defaultedOnCreate'] && ! $field['nillable'];
	}

	/**
	 * Get the list.
	 */
	public function get_modules() {
		return array(
			'Account'     => esc_html__( 'Account', 'everest-forms-salesforce' ),
			'Campaign'    => esc_html__( 'Campaign', 'everest-forms-salesforce' ),
			'Case'        => esc_html__( 'Case', 'everest-forms-salesforce' ),
			'Contact'     => esc_html__( 'Contact', 'everest-forms-salesforce' ),
			'Lead'        => esc_html__( 'Lead', 'everest-forms-salesforce' ),
			'Opportunity' => esc_html__( 'Opportunity', 'everest-forms-salesforce' ),
			'Product2'    => esc_html__( 'Product', 'everest-forms-salesforce' ),
		);
	}

	/**
	 * Get Connection.
	 *
	 * @return void
	 */
	public function get_connection() {
		if ( is_wp_error( $this->client ) ) {
			return $this->client;
		}
		return $this->connection;
	}
}
