<?php
/**
 * EverestForms Salesforce Auth Class.
 *
 * @package EverestForms\Salesforce\Api
 * @since  1.0.0
 */

namespace EverestForms\Salesforce\Api;

use League\OAuth2\Client\Provider\Exception\IdentityProviderException;
use League\OAuth2\Client\Provider\GenericProvider;
use League\OAuth2\Client\Token\AccessToken;
use Exception;
use WP_Error;

defined( 'ABSPATH' ) || exit;

/**
 * Class Auth.
 */
class Auth {

	/**
	 * Scopes that we need for authentication.
	 *
	 * @since 1.0.0
	 */
	const SCOPES = array(
		'api',
		'refresh_token',
	);

	/**
	 * Scopes Sepetator
	 *
	 * @since 1.0.0
	 */
	const SCOPES_SEPERATOR = '';

	/**
	 * Prompts that we need for authentication.
	 *
	 * @since 1.0.0
	 */
	const PROMPTS = array(
		'login',
		'consent',
	);

	/**
	 * Api error.
	 *
	 * @since 1.0.0
	 *
	 * @var \WP_Error
	 */
	protected $error;


	/**
	 * Api client.
	 *
	 * @since 1.0.0
	 */
	protected $client = null;


	/**
	 * API end point.
	 *
	 * @var string
	 */
	protected $endpoint = 'https://login.salesforce.com';


	/**
	 * Connection.
	 *
	 * Contains All the information of a connection.
	 *
	 * @var array
	 */
	protected $connection = array();


	/**
	 * Create a new Auth instance
	 *
	 * @param string $connection Connection Array.
	 */
	public function __construct( $connection ) {

		// Initialize WP_Error.
		$this->error = new WP_Error();

		// Set Connection.
		$this->connection = $connection;
	}

	/**
	 * Get Client.
	 *
	 * @return void
	 */
	public function get_client() {

		// Return Client if exists.
		if ( ! empty( $this->client ) ) {
			return $this->client;
		}

		// Return Error With No Credentials
		if ( ! $this->has_credentials() ) {
			return $this->client;
		}

		// Get New Client
		$this->client = new GenericProvider(
			array(
				'clientId'                => $this->connection['client_id'],
				'clientSecret'            => $this->connection['client_secret'],
				'scopes'                  => self::SCOPES,
				'scopeSeparator'          => self::SCOPES_SEPERATOR,
				'urlAuthorize'            => $this->get_authorization_url(),
				'redirectUri'             => $this->get_redirect_uri(),
				'urlResourceOwnerDetails' => $this->get_owner_details_url(),
				'urlAccessToken'          => $this->get_access_token_url(),
			)
		);

		// Return Client if token is valid.
		if ( $this->is_valid_token() ) {
			return $this->client;
		}

		// Regeberate Token.
		$this->regenerate_token();

		if ( $this->error->has_errors() ) {
			return $this->error;
		}

		return $this->client;
	}

	/**
	 * Token process.
	 *
	 * @since 1.0.0
	 */
	protected function regenerate_token() {

		if (
			$this->is_auth_required() &&
			! empty( $this->connection['auth_code'] )
		) {
			// When we have auth code instead of access token.
			$this->generate_token();

		} else {
			$this->refresh_token();
		}
	}

	/**
	 * Generate an access token.
	 *
	 * @since 1.0.0
	 */
	protected function generate_token() {
		// Try to get an access token using the authorization code grant.
		try {
			$token = $this->client->getAccessToken(
				'authorization_code',
				array( 'code' => $this->connection['auth_code'] )
			);

			// Update access/refresh code on connection.
			$this->update_connection( $token );

		} catch ( IdentityProviderException $e ) {
			$response = $e->getResponseBody();
			$this->error->add(
				'auth_provider_error',
				/* translators: %s - response error description. */
				sprintf( esc_html__( 'Salesforce Integration: account connection failed (%s).', 'everest-forms-salesforce' ), rtrim( $response['error_description'], '.' ) ),
				$this->connection['connection_id']
			);

		} catch ( Exception $e ) {
			$this->error->add(
				'auth_general_error',
				/* translators: %s - response error description. */
				sprintf( esc_html__( 'Salesforce Integration: account connection failed (%s).', 'everest-forms-salesforce' ), rtrim( $e->getMessage(), '.' ) ),
				$this->connection['connection_id']
			);
		}

		// Now empty auth code.
		$this->update_auth_code( '' );
	}

	/**
	 * Generate a refresh token.
	 *
	 * @since 1.0.0
	 */
	protected function refresh_token() {
		$access_token = $this->get_access_token();
		if ( empty( $access_token ) ) {
			$this->error->add(
				'empty_access_token',
				esc_html__( 'Salesforce Integration: account connection update failed (no access token found).', 'everest-forms-salesforce' ),
				$this->connection['connection_id']
			);
			return;
		}

		if ( empty( $access_token->getRefreshToken() ) ) {
			$this->error->add(
				'empty_refresh_token',
				esc_html__( 'Salesforce Integration: account connection update failed (no refresh token found).', 'everest-forms-salesforce' ),
				$this->connection['connection_id']
			);
			return;
		}

		try {
			$new_access_token = $this->client->getAccessToken(
				'refresh_token',
				array( 'refresh_token' => $access_token->getRefreshToken() )
			);

			if ( $this->update_connection( $new_access_token ) ) {
				$this->error->add(
					'auth_success',
					esc_html__( 'Salesforce Integration: account connection updated successfully.', 'everest-forms-salesforce' ),
					$this->connection['connection_id']
				);
			}
		} catch ( IdentityProviderException $e ) {
			$response = $e->getResponseBody();

			$this->error->add(
				'auth_provider_error',
				/* translators: %s - response error description. */
				sprintf( esc_html__( 'Salesforce Integration: account connection update failed (%s).', 'everest-forms-salesforce' ), rtrim( $response['error_description'], '.' ) ),
				$this->connection['connection_id']
			);

		} catch ( Exception $e ) {
			$this->error->add(
				'auth_general_error',
				/* translators: %s - response error description. */
				sprintf( esc_html__( 'Salesforce Integration: account connection update failed (%s).', 'everest-forms-salesforce' ), rtrim( $e->getMessage(), '.' ) ),
				$this->connection['connection_id']
			);
		}
	}

	/**
	 * Update access/refresh token.
	 *
	 * @param AccessToken $token AccessToken Object.
	 * @return void
	 */
	protected function update_connection( $token ) {
		$access_token  = $token->getToken();
		$refresh_token = $token->getRefreshToken();

		if ( ! empty( $access_token ) ) {
			$this->connection['access_token'] = $access_token;
		}

		if ( ! empty( $refresh_token ) ) {
			$this->connection['refresh_token'] = $refresh_token;
		}

		$data = $token->getValues();

		if ( ! empty( $data['instance_url'] ) ) {
			$this->connection['instance_url'] = $data['instance_url'];
		}

		if ( $this->connection['connection_id'] ) {
			$this->connection['auth_code'] = ''; // Empty auth code.
			$connected_accounts            = get_option( 'everest_forms_integrations', array() );
			$connected_accounts['salesforce'][ $this->connection['connection_id'] ] = $this->connection;
			update_option( 'everest_forms_integrations', $connected_accounts );
		}
	}

	/**
	 * Whether connection has Client ID and Client Secret or not.
	 * Both options are required.
	 *
	 * @since 1.0.0
	 *
	 * @return bool
	 */
	protected function has_credentials() {
		return ! empty( $this->connection['client_id'] ) && ! empty( $this->connection['client_secret'] );
	}

	/**
	 * Check if access token is valid.
	 *
	 * @since 1.0.0
	 *
	 * @return bool Whether access token exists and we can retrieve an external data.
	 */
	protected function is_valid_token() {
		$access_token = $this->get_access_token();
		if ( ! ( $access_token instanceof AccessToken ) ) {
			return false;
		}

		// Try to receive user details data.
		if ( empty( $this->get_owner_details( $access_token ) ) ) {
			return false;
		}
		return true;
	}

	/**
	 * Init and get the AccessToken object.
	 *
	 * @since 1.0.0
	 *
	 * @return \League\OAuth2\Client\Token\AccessToken|null
	 */
	public function get_access_token() {
		return ! empty( $this->connection['access_token'] ) ? new AccessToken( $this->connection ) : null;
	}

	/**
	 * Check if it requires authentication.
	 *
	 * @return bool
	 */
	public function is_auth_required() {
		return empty( $this->connection['access_token'] ) || empty( $this->connection['refresh_token'] );
	}

	/**
	 * Update an authorization code.
	 *
	 * @since 1.0.0
	 *
	 * @param string $code Authorization code.
	 */
	protected function update_auth_code( $code ) {
		$this->connection['auth_code'] = $code;
	}

	/**
	 * Get redirect url.
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	public function get_redirect_uri() {
		return esc_url_raw( rtrim( site_url() , "/" ) . '?evf-integration=salesforce' );
	}

	/**
	 * Get access token url to retrieve token.
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	public function get_access_token_url() {
		return $this->endpoint . '/services/oauth2/token';
	}

	/**
	 * Get authorization url to begin OAuth flow.
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	public function get_authorization_url() {
		return $this->endpoint . '/services/oauth2/authorize';
	}

	/**
	 * Get provider url to fetch user details.
	 *
	 * @since 1.0.0
	 *
	 * @return string
	 */
	public function get_owner_details_url() {
		return $this->endpoint . '/services/oauth2/userinfo';
	}


	/**
	 * Request and return the resource owner of given access token.
	 *
	 * @since 1.0.0
	 *
	 * @param \League\OAuth2\Client\Token\AccessToken $access_token Access token.
	 *
	 * @return array
	 */
	protected function get_owner_details( $access_token ) {

		$result = array();

		try {
			$resource_owner = $this->client->getResourceOwner( $access_token );
			$result         = $resource_owner->toArray();

			// Resource error.
			if ( empty( $result['user_id'] ) ) {
				$this->error->add(
					'resource_owner_error',
					esc_html__( 'Salesforce Integration: data retrieval failed (resource owner error).', 'everest-forms-salesforce' ),
					$this->connection['connection_id']
				);
				$result = array();
			}
		} catch ( IdentityProviderException $e ) {
			$response = $e->getResponseBody();

			$this->error->add(
				'resource_owner_provider_error',
				/* translators: %s - response error description. */
				sprintf( esc_html__( 'Salesforce Integration: data retrieval failed (%s).', 'everest-forms-salesforce' ), rtrim( $response['error_description'], '.' ) ),
				$this->connection['connection_id']
			);

		} catch ( Exception $e ) {
			$this->error->add(
				'resource_owner_general_error',
				/* translators: %s - response error description. */
				sprintf( esc_html__( 'Salesforce Integration: data retrieval failed (%s).', 'everest-forms-salesforce' ), rtrim( $e->getMessage(), '.' ) ),
				$this->connection['connection_id']
			);
		}

		return $result;
	}
}
