<?php
/**
 * Salesforce Marketing Class.
 *
 * @package EverestForms\Salesforce\Builder\SalesforceMarketing
 * @since   1.0.0
 */

namespace EverestForms\Salesforce\Builder;

use EverestForms\Salesforce\Api\Api;
use EverestForms\Salesforce\Salesforce;
use EverestForms\Salesforce\Settings\SalesforceIntegration;
use Mustache_Logger_StreamLogger;

defined( 'ABSPATH' ) || exit;


/**
 * Salesforce Marketting class.
 */
class Marketing extends \EVF_Email_Marketing {
	/**
	 * Constructor.
	 */
	public function __construct() {
		$this->id            = 'salesforce';
		$this->name          = __( 'Salesforce', 'everest-forms-salesforce' );
		$this->icon          = plugins_url( '/assets/images/salesforce.png', EVF_SALESFORCE_PLUGIN_FILE );
		$this->authorize_url = 'https://login.salesforce.com/services/oauth2/authorize';

		parent::__construct();

		add_filter( 'everest_forms_salesforce_add_account_html', array( $this, 'outputs_authentication_html' ) );
	}

	/**
	 * Process and submit entry to provider.
	 *
	 * @param array $fields    List of form fields.
	 * @param array $entry     User submitted data.
	 * @param array $form_data Prepared form settings.
	 * @param int   $entry_id  Entry ID.
	 */
	public function process_feed( $fields, $entry, $form_data, $entry_id = 0 ) {

		// Only run if this form has connections for this provider.
		if ( empty( $form_data['integrations'][ $this->id ] ) ) {
			return;
		}

		// Fire for each connection.
		foreach ( $form_data['integrations'][ $this->id ] as $connection ) :

			// Check for conditional logic.
			if ( isset( $connection['conditional_logic']['status'] ) ) {
				$con_field_select          = isset( $connection['conditional_logic']['field_select'] ) ? $connection['conditional_logic']['field_select'] : '';
				$con_field_condition       = isset( $connection['conditional_logic']['condition'] ) ? $connection['conditional_logic']['condition'] : '';
				$con_field_input_choice    = isset( $connection['conditional_logic']['input_choice'] ) ? $connection['conditional_logic']['input_choice'] : '';
				$con_field_multiple_choice = isset( $connection['conditional_logic']['multiple_choice'] ) ? $connection['conditional_logic']['multiple_choice'] : '';
				$con_field_country_choice  = isset( $connection['conditional_logic']['country_choice'] ) ? $connection['conditional_logic']['country_choice'] : '';

				if ( ! empty( $con_field_input_choice ) ) {
					$con_field_choice = $con_field_input_choice;
				} elseif ( ! empty( $con_field_multiple_choice ) ) {
					$con_field_choice = $con_field_multiple_choice;
				} elseif ( ! empty( $con_field_country_choice ) ) {
					$con_field_choice = $con_field_country_choice;
				} else {
					$con_field_choice = '';
				}

				if ( 'is' === $con_field_condition ) {
					if ( isset( $entry['form_fields'][ $con_field_select ] ) ) {
						if ( is_array( $entry['form_fields'][ $con_field_select ] ) ) {
							if ( ! in_array( $con_field_choice, $entry['form_fields'][ $con_field_select ], true ) ) {
								continue;
							}
						} else {
							if ( $entry['form_fields'][ $con_field_select ] !== $con_field_choice ) {
								continue;
							}
						}
					} else {
						continue;
					}
				} elseif ( 'is_not' === $con_field_condition ) {
					if ( isset( $entry['form_fields'][ $con_field_select ] ) ) {
						if ( is_array( $entry['form_fields'][ $con_field_select ] ) ) {
							if ( in_array( $con_field_choice, $entry['form_fields'][ $con_field_select ], true ) ) {
								continue;
							}
						} else {
							if ( $entry['form_fields'][ $con_field_select ] === $con_field_choice ) {
								continue;
							}
						}
					}
				}
			}

			// Setup basic data.
			$account_id = $connection['account_id'];
			$list_id    = $connection['list_id'];
			$api        = $this->api_connect( $account_id );
			$data       = array();

			// Bail if there is any sort of issues with the API connection.
			if ( is_wp_error( $api->get_connection() ) ) {
				continue;
			}

			$salesforce_fields = $this->api->get_fields( $list_id );

			$data = array();
			// Setup merge fields.
			if ( ! empty( $connection['fields'] ) ) {
				foreach ( $connection['fields'] as $name => $merge_field ) {
					$merge_field = explode( '.', $merge_field );
					$id          = $merge_field[0]; // evf Field ID.
					$key         = ! empty( $merge_field[1] ) ? $merge_field[1] : 'value';
					$type        = ! empty( $merge_field[2] ) ? $merge_field[2] : 'text'; // MC merge field type.

					// Check if mapped form field has a value.
					if ( empty( $fields[ $id ][ $key ] ) ) {
						continue;
					}

					$value = $fields[ $id ][ $key ];

					// Format edge cases pre-API call.
					if ( is_array( $value ) ) {
						switch ( $fields[ $id ]['type'] ) {
							case 'radio':
							case 'payment-radio':
								$value = ! empty( $value['label'] ) ? $value['label'] : '';
								break;

							case 'checkbox':
							case 'payment-checkbox':
								$value = ! empty( $value['label'] ) && is_array( $value['label'] ) ? implode( ', ', (array) $value['label'] ) : $value['label'];
								break;

							case 'MailingAddress':
								$value = str_replace( '<br />', ', ', nl2br( $value ) );
								break;
							case 'country':
								$value = apply_filters( 'everest_forms_plaintext_field_value', $value['country_code'], $value, $form_data, 'email-plain' );
								break;
							default:
								$value = implode( ' ', $value );
								break;
						}
					}

					if ( ! empty( $salesforce_fields['required'][ $name ] ) ) {
						$data = $this->format_data( $data, $name, $value, $salesforce_fields['required'][ $name ], $list_id, $fields );
					}
				}
			}

			// Setup Other fields.
			if ( isset( $connection['custom_field_value'] ) && ! empty( $connection['custom_field_value'] ) ) {
				foreach ( $connection['custom_field_value'] as $name => $custom_field ) {
					$custom_field = explode( '.', $custom_field );
					$id           = $custom_field[0]; // evf Field ID.
					$key          = ! empty( $custom_field[1] ) ? $custom_field[1] : 'value';
					$type         = ! empty( $custom_field[2] ) ? $custom_field[2] : 'text'; // MC merge field type.

					// Check if mapped form field has a value.
					if ( empty( $fields[ $id ][ $key ] ) ) {
						continue;
					}

					$value = $fields[ $id ][ $key ];

					// Format edge cases pre-API call.
					if ( is_array( $value ) ) {
						switch ( $fields[ $id ]['type'] ) {
							case 'radio':
							case 'payment-radio':
								$value = ! empty( $value['label'] ) ? $value['label'] : '';
								break;

							case 'checkbox':
							case 'payment-checkbox':
								$value = ! empty( $value['label'] ) && is_array( $value['label'] ) ? '||' . implode( '||', (array) $value['label'] ) . '||' : $value['label'];
								break;
							case 'MailingAddress':
								$value = str_replace( '<br />', ', ', nl2br( $value ) );
								break;
							default:
								$value = implode( ' ', $value );
								break;
						}
					}
					$name = ! empty( $connection['custom_field'][ $name ] ) ? $connection['custom_field'][ $name ] : '';
					if ( ! empty( $salesforce_fields['optional'][ $name ] ) ) {
						$data = $this->format_data( $data, $name, $value, $salesforce_fields['optional'][ $name ], $list_id, $fields );
					}
				}
			}
			// Send to API.
			$response = $api->create_object( $list_id, $data );

			if ( is_wp_error( $response ) ) {
				Salesforce::log( $response->get_error_messages(), 'error' );
				return;
			}

			if ( ! isset( $response['success'] ) || true !== $response['success'] ) {
				$error_msg = ! empty( $response[0]['message'] ) ? $response[0]['message'] : __( 'Error while creating subscription.', 'everest-forms-salesforce' );
				Salesforce::log( $error_msg, 'error' );
			}
		endforeach;
	}

	/**
	 * Format Dat
	 *
	 * @param mixed $data Data Array.
	 * @param mixed $name Field Name
	 * @param mixed $value Field Value.
	 * @param mixed $field Field Array.
	 * @return void
	 */
	public function format_data( $data, $name, $value, $field, $list_id, $fields ) {
		$last_name = '';
		foreach ( $fields as $field_key => $field_value ) {
			if ( 'last-name' === $field_value['type'] ) {
				$last_name = $field_value['value'];
			}
		}
		if ( empty( $field['compound_fields'] ) ) {
			$data[ $name ] = $value;
			return $data;
		}
		switch ( $name ) {
			case 'Name':
				if ( 'Account' === $list_id ) {
					$data[ $name ] = $value;
					break;
				}

				$value             = explode( ' ', trim( $value ) );
				$data['FirstName'] = $value[0];
				unset( $value[0] );
				if ( empty( $value ) ) {
					$data['LastName'] = $last_name;
				} else {
					$data['LastName'] = implode( ' ', $value );
				}

				break;
			case 'MailingAddress':
				$address                   = explode( ',', $value );
				$firstaddress              = ! empty( $address[0] ) ? explode( '<br />', nl2br( $address[0] ) ) : '';
				$secondaddress             = ! empty( $address[1] ) ? explode( '<br />', nl2br( $address[1] ) ) : '';
				$data['MailingStreet']     = ! empty( $firstaddress[0] ) ? trim( $firstaddress[0] ) : '';
				$data['MailingStreet']    .= ! empty( $firstaddress[1] ) ? trim( $firstaddress[1] ) : '';
				$data['MailingCity']       = ! empty( $firstaddress[2] ) ? trim( $firstaddress[2] ) : '';
				$data['MailingState']      = ! empty( $secondaddress[0] ) ? trim( $secondaddress[0] ) : '';
				$data['MailingPostalCode'] = ! empty( $secondaddress[1] ) ? trim( $secondaddress[1] ) : '';
				$data['MailingCountry']    = ! empty( $secondaddress[2] ) ? trim( $secondaddress[2] ) : '';
				break;
			default:
				$data[ $name ] = $value;
				break;
		}
		return $data;
	}


	/**
	 * Connect to API.
	 *
	 * @param  string $account_id  Account ID for Mailchimp.
	 * @return mixed array or error object
	 */
	public function api_connect( $connection_id ) {
		if ( ! empty( $this->api ) && $connection_id === $this->connection_id ) {
			return $this->api;
		} else {
			$providers = get_option( 'everest_forms_integrations' );
			if ( ! empty( $providers[ $this->id ][ $connection_id ] ) ) {
				$connection          = $providers[ $this->id ][ $connection_id ];
				$this->connection_id = $connection_id;
				$this->api           = new Api( $connection );
				return $this->api;
			} else {
				return $this->error( __( 'API connection error', 'everest-forms-salesforce' ) );
			}
		}
	}

	/**
	 * Get Integration account lists.
	 *
	 * @param string $connection_id Connection ID for Salesforce.
	 * @param string $account_id    Account ID for Salesforce.
	 *
	 * @return mixed array or error object
	 */
	public function api_lists( $connection_id = '', $account_id = '' ) {
		$this->api_connect( $account_id );
		try {
			$lists = $this->api->get_modules();
			if ( ! empty( $lists ) ) {
				$list_array = array();
				foreach ( $lists as $key => $list ) {
					$list_array[ $key ] = array(
						'id'   => $key,
						'name' => isset( $list ) ? trim( $list ) : __( 'Unknown List', 'everest-forms-salesforce' ),
					);
				}
				return $list_array;
			} else {
				$error_msg = __( 'Salesforce API list error: No lists found', 'everest-forms-salesforce' );
				return $this->error( $error_msg );
			}
		} catch ( \Exception $e ) {
			evf_get_logger(
				__( 'Salesforce API error', 'everest-forms-salesforce' ),
				$e->getMessage(),
				array(
					'type' => array( 'Integration', 'error' ),
				)
			);
		}
	}

	/**
	 * Fetch Integration account list fields.
	 *
	 * @param string $connection_id Connection ID for fetching the connection object.
	 * @param string $account_id    Account ID for fetching api fields.
	 * @param string $list_id       List id for fetching api fields.
	 *
	 * @return mixed array or error object
	 */
	public function fetch_api_fields( $connection_id = '', $account_id = '', $list_id = '' ) {
		$this->api_connect( $account_id );
		try {
			$object_name = isset( $_POST['list_id'] ) ? sanitize_text_field( wp_unslash( $_POST['list_id'] ) ) : $list_id;
			$fields      = $this->api->get_fields( $object_name );
			if ( ! empty( $fields ) ) {
				$required = array_column( $fields['required'], 'label', 'name' );
				$optional = array_column( $fields['optional'], 'label', 'name' );

				asort( $optional );

				$fields['required'] = $required;
				$fields['optional'] = $optional;
			}
			$fld = array();
			if ( ! is_wp_error( $fields ) && ! empty( $fields ) ) {
				foreach ( $fields['required'] as $key => $value ) {
					$fld[ $key ] = array(
						'id'         => $key,
						'name'       => isset( $value ) ? trim( $value ) : __( 'Unknown Name', 'everest-forms-salesforce' ),
						'field_type' => 'text',
						'tag'        => $key,
					);

				}
			}
			return $fld;
		} catch ( Exception $e ) {

			/* translators: %s: API Authentication Error. */
			Salesforce::log( sprintf( __( 'Salesforce API error: %s', 'everest-forms-salesforce' ), $e->getMessage() ), 'error' );

			/* translators: %s: API Authentication Error. */
			$error_msg = sprintf( __( 'Salesforce API fields error: %s', 'everest-forms-salesforce' ), $e->getMessage() );

			return $this->error( $error_msg );
		}
	}

	/**
	 * Fetch Integration account list fields.
	 *
	 * @param string $connection_id Connection ID for fetching the connection object.
	 * @param string $account_id    Account ID for fetching api fields.
	 * @param string $list_id       List id for fetching api fields.
	 */
	public function api_custom_field( $connection_id = '', $account_id = '', $list_id = 'Account' ) {
		$this->api_connect( $account_id );
		try {
			$object_name = isset( $_POST['list_id'] ) ? sanitize_text_field( wp_unslash( $_POST['list_id'] ) ) : $list_id;
			$fields      = $this->api->get_fields( $object_name );
			if ( ! empty( $fields ) ) {
				$required = array_column( $fields['required'], 'label', 'name' );
				$optional = array_column( $fields['optional'], 'label', 'name' );

				asort( $optional );

				$fields['required'] = $required;
				$fields['optional'] = $optional;
			}
			$fld = array();
			if ( ! is_wp_error( $fields ) && ! empty( $fields ) ) {
				foreach ( $fields['optional'] as $key => $value ) {
					$fld[ $key ] = array(
						'id'         => $key,
						'name'       => isset( $value ) ? trim( $value ) : __( 'Unknown Merge Variables', 'everest-forms-salesforce' ),
						'field_type' => 'text',
						'tag'        => '',
					);
				}
			}
			return $fld;
		} catch ( Exception $e ) {

			/* translators: %s: API Authentication Error. */
			Salesforce::log( sprintf( __( 'Salesforce API error: %s', 'everest-forms-salesforce' ), $e->getMessage() ), 'error' );

			/* translators: %s: API Authentication Error. */
			$error_msg = sprintf( __( 'Salesforce API fields error: %s', 'everest-forms-salesforce' ), $e->getMessage() );

			return $this->error( $error_msg );
		}
	}

	/**
	 * Output Authentication HTML.
	 *
	 * @param string $output Output HTML.
	 */
	public function outputs_authentication_html( $output ) {
		ob_start();
		?>
		<a href="<?php echo $this->authorize_url; ?>" class="everest-forms-btn everest-forms-btn-primary everest-forms-integration-open-window" data-source="<?php echo esc_attr( $this->id ); ?>" data-redirect-uri="<?php echo esc_url_raw( rtrim( site_url(), '/' ) . '?page=evf-integration=salesforce' ); ?>"><?php echo esc_html__( 'Get Access Code', 'everest-forms-salesforce' ); ?></a>
		<div class="everest-forms-hidden evf-salesforce-account">
			<input type="text" placeholder="Salesforce Account Nickname" id="evf-salesforce-nickname" data-name="label" class="everest_forms_salesforce_account_name" required>
			<input type="text" data-name="auth_code" id="evf-salesforce-auth-code" class="everest_forms_salesforce_auth_code" placeholder="<?php esc_attr_e( 'Enter Salesforce Access Code', 'everest-forms-salesforce' ); ?>" required>
		</div>
		<button type="submit" style="display:none" class="everest-forms-btn everest-forms-btn-primary evf-salesforce-connect" data-source="<?php echo esc_attr( $this->id ); ?>"><?php esc_html_e( 'Connect to Salesforce', 'everest-forms-salesforce' ); ?></button>
		<?php
		$output = ob_get_clean();
		return $output;
	}

	/**
	 * Authenticate with the Integration API.
	 *
	 * @param array  $data    Data passed for API authorization.
	 * @param string $form_id Form ID.
	 * @throws \Exception Exception.
	 * @return mixed id or error object
	 */
	public function authorize_api( $data = array(), $form_id = '' ) {
		$connection_args = array(
			'client_id'     => trim( $data['client_id'] ),
			'client_secret' => trim( $data['client_secret'] ),
			'label'         => trim( $data['label'] ),
			'auth_code'     => trim( $data['auth_code'] ),
			'connection_id' => uniqid(),
			'status'        => true,
			'date'          => time(),
			'instance_url'  => '',

		);
		$api = new Api( $connection_args );

		$connection = $api->get_connection();

		if ( is_wp_error( $connection ) ) {
			return $this->error( __( 'Could not verify API key', 'everest-forms-pipedrive' ) );
		}
		return $connection['connection_id'];
	}

	/**
	 * Integration account lists groups HTML.
	 *
	 * @param string $connection_id Connection identifier for connected accounts.
	 * @param array  $connection    Connection data object.
	 *
	 * @return WP_Error|string
	 */
	public function output_groups( $connection_id = '', $connection = array() ) {
		return '';
	}
}
