<?php
/**
 * Main plugin class.
 *
 * @package EverestForms\Salesforce
 * @since   1.0.0Google Drive
 */

namespace EverestForms\Salesforce;

use EverestForms\Salesforce\Builder\Marketing;
use EverestForms\Salesforce\Settings\Settings;

/**
 * Main plugin class.
 *
 * @since 1.0.0
 */
class Salesforce {

	/**
	 * The single instance of the class.
	 *
	 * @var object
	 *
	 * @since 1.0.0
	 */
	protected static $instance;


	/**
	 * Logger Instance
	 *
	 * @var object
	 */
	public static $log = null;

	/**
	 * Cloning is forbidden.
	 *
	 * @since 1.0.0
	 */
	public function __clone() {
		_doing_it_wrong( __FUNCTION__, esc_html__( 'Cloning is forbidden.', 'everest-forms-salesforce' ), '1.0.0' );
	}

	/**
	 * Unserializing instances of this class is forbidden.
	 *
	 * @since 1.0.0
	 */
	public function __wakeup() {
		_doing_it_wrong( __FUNCTION__, esc_html__( 'Unserializing instances of this class is forbidden.', 'everest-forms-salesforce' ), '1.0.0' );
	}

	/**
	 * Main plugin class instance.
	 *
	 * @since 1.0.0
	 *
	 * Ensures only one instance of the plugin is loaded or can be loaded.
	 *
	 * @return object Main instance of the class.
	 */
	final public static function instance() {
		if ( is_null( static::$instance ) ) {
			static::$instance = new static();
		}
		return static::$instance;
	}

	/**
	 * Plugin Constructor.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		add_action( 'init', array( $this, 'load_plugin_textdomain' ) );
		add_filter( 'plugin_row_meta', array( $this, 'plugin_row_meta' ), 20, 2 );

		// Checks with Everest Forms is installed.
		if ( defined( 'EVF_VERSION' ) && version_compare( EVF_VERSION, '1.9.3', '>=' ) ) {

			// Checks with Everest Forms Pro is installed.
			if ( defined( 'EFP_VERSION' ) && version_compare( EFP_VERSION, '1.5.8', '>=' ) ) {
				// Hooks.
				add_action( 'everest_forms_init', array( $this, 'plugin_updater' ) );
				add_action( 'everest_forms_init', array( $this, 'salesforce_init' ) );
				add_filter( 'everest_forms_integrations', array( $this, 'add_integration' ) );
			} else {
				add_action( 'admin_notices', array( $this, 'everest_forms_pro_missing_notice' ) );
			}
		} else {
			add_action( 'admin_notices', array( $this, 'everest_forms_missing_notice' ) );
		}
	}



	/**
	 * Init Salesforce - Logic begins.
	 */
	public function salesforce_init() {
		new Settings();
		new Marketing();
	}

	/**
	 * Plugin Updater.
	 *
	 * @since 1.0.0
	 */
	public function plugin_updater() {
		if ( class_exists( 'EVF_Plugin_Updater' ) ) {
				\EVF_Plugin_Updater::updates( EVF_SALESFORCE_PLUGIN_FILE, 226648, EVF_SALESFORCE_VERSION );
		}
	}



	/**
	 * Register Salesforce integration.
	 *
	 * @param array $integrations List of integrations.
	 */
	public function add_integration( $integrations ) {
		$integrations[] = 'EverestForms\Salesforce\Settings\Integration';
		return $integrations;
	}


	/**
	 * Load Localization files.
	 *
	 * Note: the first-loaded translation file overrides any following ones if the same translation is present.
	 *
	 * Locales found in:
	 *      - WP_LANG_DIR/everest-forms-salesforce/everest-forms-salesforce-LOCALE.mo
	 *      - WP_LANG_DIR/plugins/everest-forms-salesforce-LOCALE.mo
	 */
	public function load_plugin_textdomain() {
		$locale = apply_filters( 'plugin_locale', get_locale(), 'everest-forms-salesforce' );

		load_textdomain( 'everest-forms-salesforce', WP_LANG_DIR . '/everest-forms-salesforce/everest-forms-salesforce-' . $locale . '.mo' );
		load_plugin_textdomain( 'everest-forms-salesforce', false, plugin_basename( dirname( EVF_SALESFORCE_PLUGIN_FILE ) ) . '/languages' );
	}

	/**
	 * Display row meta in the Plugins list table.
	 *
	 * @since 1.0.0
	 *
	 * @param  array  $plugin_meta Plugin Row Meta.
	 * @param  string $plugin_file Plugin Base file.
	 * @return array
	 */
	public function plugin_row_meta( $plugin_meta, $plugin_file ) {
		if ( plugin_basename( EVF_SALESFORCE_PLUGIN_FILE ) === $plugin_file ) {
			$new_plugin_meta = array(
				'docs' => '<a href="' . esc_url( 'https://docs.everestforms.net/docs/salesforce/' ) . '" aria-label="' . esc_attr__( 'View Everest Forms Salesforce documentation', 'everest-forms-salesforce' ) . '">' . esc_html__( 'Docs', 'everest-forms-salesforce' ) . '</a>',
			);

			return array_merge( $plugin_meta, $new_plugin_meta );
		}

		return (array) $plugin_meta;
	}

	/**
	 * Everest Forms fallback notice.
	 *
	 * @since 1.0.0
	 */
	public function everest_forms_missing_notice() {
		/* translators: %s: everest-forms version */
		echo '<div class="error notice is-dismissible"><p>' . sprintf( esc_html__( 'Everest Forms - Salesforce requires at least %s or later to work!', 'everest-forms-salesforce' ), '<a href="https://everestforms.net/" target="_blank">' . esc_html__( 'Everest Forms 1.9.3', 'everest-forms-salesforce' ) . '</a>' ) . '</p></div>';
	}

	/**
	 * Everest Forms Pro fallback notice.
	 *
	 * @since 1.0.0
	 */
	public function everest_forms_pro_missing_notice() {
		/* translators: %s: everest-forms-Salesforce version */
		echo '<div class="error notice is-dismissible"><p>' . sprintf( esc_html__( 'Everest Forms - Salesforce depends on the last version of %s or later to work!', 'everest-forms-salesforce' ), '<a href="https://everestforms.net/" target="_blank">' . esc_html__( 'Everest Forms Pro 1.5.8', 'everest-forms-salesforce' ) . '</a>' ) . '</p></div>';
	}

	/**
	 * Logging method.
	 *
	 * @param string $message Log message.
	 * @param string $level Optional. Default 'info'. Possible values:
	 *                      emergency|alert|critical|error|warning|notice|info|debug.
	 */
	public static function log( $message, $level = 'info' ) {
		if ( empty( self::$log ) ) {
			self::$log = evf_get_logger();
		}
		self::$log->log( $level, $message, array( 'source' => 'salesforce' ) );
	}
}
